#!/usr/bin/python3

from guimadeeasy.calculator import *
from guimadeeasy.notepad import *
from guimadeeasy.todo import *
from guimadeeasy.wheather import *
from guimadeeasy.snake import *
from guimadeeasy.awesomeclock import *
